!-----------------------------egs5_rk1.f--------------------------------
! Version: 051219-1435
! Read the input data tables of K1 vs R and Z, and construct
! PWL of K1 vs. E for all materials in this problem
!-----------------------------------------------------------------------
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12

      subroutine rk1

      implicit none

      include 'include/egs5_h.f'         ! COMMONs required by EGS5 code
      include 'include/egs5_useful.f'
      include 'include/egs5_brempr.f'
      include 'include/egs5_thresh.f'
      include 'include/egs5_elecin.f'
      include 'include/egs5_media.f'
      include 'include/egs5_misc.f'
      include 'include/egs5_scpw.f'
      
      real*8                                           ! Local variables
     * ek0k1(40,20),
     * slopek1(40,20), bk1(40,20), 
     * dlowk1(40,20), dhighk1(40,20),
     * k1low(40,20), k1high(40,20),
     * z2rhok1(20), z2k1(20),
     * rhok1(20)

      integer nmatk1,nek1(20),z2idx(20)

      real*8
     * rdum, lcharD, eke, elke, delke, z2, z2rho, 
     * rhofact(2), pznorm, zfrac, frace, k1ez(2),
     * k1z(2), k1new, k1old, elkeold, scpe, scpp, scprat, 
     * c1, c2

      integer nz2rho, iz2rho, idx, iek1, iedx, niek, neke
      integer i,im,ie,j,n, m

      open(UNIT=17,FILE='data/K1.dat',STATUS='old')

!  Read the K1 data for this element, and then compute the
!  interpolated value for the characteristic distance at
!  each energy

      read(17,*) nmatk1
      do i = 1, nmatk1
        read(17,*) z2rhok1(i), rdum

        !  for gases we need z2, but z2 and z2rho are 
        !  not both monotonic, so compute an index
        z2k1(i) = rdum**2
        z2idx(i) = i
        rhok1(i) = z2rhok1(i)/z2k1(i)
        if(i.gt.1) then
          if(z2k1(i) .lt. z2k1(i-1)) then
            rdum = z2k1(i)
            z2k1(i) = z2k1(i-1)
            z2k1(i-1) = rdum
            z2idx(i) = i-1
            z2idx(i-1) = i
          endif
        endif

        read(17,*) nek1(i)
        do j = nek1(i), 1, -1
          read(17,*) ek0k1(j,i), slopek1(j,i), bk1(j,i), 
     *              dlowk1(j,i),k1low(j,i),dhighk1(j,i),k1high(j,i)
        end do
      end do

      close(17)

!  load K1 as a function of E for all elements.  Get Z^2 x rho,
!  then loop over energies and do two-way interpolation

      do 10 im = 1, nmed

        if(charD(im).eq.0.d0) go to 10
        lcharD = dlog(charD(im))

        pznorm = 0.d0
        z2 = 0.d0
        do ie = 1, nne(im)
          z2 = z2 + pz(im,ie) * zelem(im,ie) * zelem(im,ie)
          pznorm = pznorm + pz(im,ie)
        end do
        z2 = z2 / pznorm
        z2rho = z2 * rhom(im)

        rhofact(1) = 1.d0
        rhofact(2) = 1.d0

        !  Low z^2 rho cases:  Use z^2 and scale by rho
        if(z2rho.lt.z2rhok1(1)) then
          if(z2.gt.z2k1(nmatk1)) then
            nz2rho = 1
            zfrac = 0.d0
            iz2rho = nmatk1
            rhofact(1) = rhom(im) / rhok1(iz2rho)
          else if(z2.lt.z2k1(1)) then
            nz2rho = 1
            zfrac = 0.d0
            iz2rho = 1
            rhofact(1) = rhom(im) / rhok1(iz2rho)
          else 
            nz2rho = 2
            call findi(z2k1,z2,nmatk1,iz2rho)
            iz2rho = z2idx(iz2rho)
            zfrac = (z2 - z2k1(iz2rho)) / 
     *               (z2k1(iz2rho+1) - z2k1(iz2rho))
            rhofact(1) = rhom(im) / rhok1(iz2rho)
            rhofact(2) = rhom(im) / rhok1(iz2rho+1)
          endif

        !  condensed matter cases:

        !  extrapolate high z^2 rho
        else if(z2rho.gt.z2rhok1(nmatk1)) then
          nz2rho = 1
          zfrac = 0.d0
          iz2rho = nmatk1
        !  interpolate
        else 
          nz2rho = 2
          call findi(z2rhok1,z2rho,nmatk1,iz2rho)
          zfrac = (z2rho - z2rhok1(iz2rho)) / 
     *             (z2rhok1(iz2rho+1) - z2rhok1(iz2rho))
        endif

        neke = meke(im)
        do j = 1,neke-1
          if(j.eq.1) then
            eke = ae(im) - RM
            elke = log(eke)
          else if(j.eq.neke-1) then
            eke = ue(im) - RM
            elke = log(eke)
          else
            elke = (j + 1 - eke0(im)) / eke1(im)
            eke = dexp(elke)
          endif

          scpe = escpw1(j,im)*elke + escpw0(j,im)
          scpp = pscpw1(j,im)*elke + pscpw0(j,im)
          scprat = scpp/scpe
          do n = 1, nz2rho
            idx = iz2rho + n - 1
            if(eke.gt.ek0k1(nek1(idx),idx)) then
              niek = 1
              frace = 0.d0
              iek1 = nek1(idx)
            else if(eke.lt.ek0k1(1,idx)) then
              niek = 1
              frace = 0.d0
              iek1 = 1
            else 
              niek = 2
              call findi(ek0k1(1,idx),eke,nek1(idx),iek1)
              frace = (eke - ek0k1(iek1,idx)) / 
     *                    (ek0k1(iek1+1,idx) - ek0k1(iek1,idx))
            endif
            do m = 1, niek
              iedx = iek1 + m - 1
              if(charD(im).gt.dhighk1(iedx,idx)) then
                k1ez(m) = k1high(iedx,idx)
              else if(charD(im).lt.dlowk1(iedx,idx)) then
                k1ez(m) = k1low(iedx,idx)
              else
                k1ez(m) = DEXP(slopek1(iedx,idx)*lcharD + bk1(iedx,idx))
              endif
            end do
            k1z(n) = k1ez(1) + frace * (k1ez(2) - k1ez(1))
            k1z(n) = k1z(n) * rhofact(n)
          end do
          k1new = (k1z(1) + zfrac * (k1z(2) - k1z(1)))

          if(j.gt.1) then
            delke = elke - elkeold
            ekini1(j,im) =  (k1new - k1old) / delke
            ekini0(j,im) = (k1old * elke - k1new * elkeold) / delke
            pkini1(j,im) = ekini1(j,im) * scprat
            pkini0(j,im) = ekini0(j,im) * scprat
          endif
          elkeold = elke
          k1old = k1new
        end do
            
        ekini1(1,im) = ekini1(2,im)
        ekini0(1,im) = ekini0(2,im)
        pkini1(1,im) = pkini1(2,im)
        pkini0(1,im) = pkini0(2,im)
        ekini1(neke,im) = ekini1(neke-1,im)
        ekini0(neke,im) = ekini0(neke-1,im)
        pkini1(neke,im) = pkini1(neke-1,im)
        pkini0(neke,im) = pkini0(neke-1,im)

10    continue

!  compute constants for user requested K1 scaling

      do j = 1, nreg
        if(k1Lscl(j).gt.0.d0 .and. k1Hscl(j).gt.0.d0) then
          im = med(j)
          c2 = (k1Lscl(j) - k1Hscl(j)) /
     *           dlog( (ae(im) - RM)/(ue(im) - RM) )
          c1 = k1Hscl(j) - dlog(ue(im) - RM) * c2
          k1Lscl(j) = c1
          k1Hscl(j) = c2
        endif 
      end do

      return
      end

!-----------------------last line of egs5_rk1.f-------------------------
